# -*- coding: utf-8 -*-
"""
Created on Wed Jan 23 12:23:53 2019

@author: Domen
"""
#import time
#import urllib2
#import base64
#import json
#import math
#import os
#import datetime

from lxml import etree
import socket
import os
from datetime import date
from py_logging.cestel_logging import log, log_exception
import logging

import base64
import threading
from generic_module import Module
import cestel_exceptions
 
class CommunicationModule(Module):
  def __init__(self, args):
    expected_params = {"port":8173, "backlog":5, "recv_buffer_size":2048, "root_path":None}
    Module.__init__(self, args, expected_params)
    self.parser = etree.XMLParser(remove_blank_text=True)
    self.num_of_clients = 0
    self.clients_lock = threading.Lock()
    self.reset_i = False
  
  def are_we_resetting(self):
    if self.reset_i:
      log(self.logger, 'Raising RestartI exception.', sev=logging.DEBUG)
      raise cestel_exceptions.RestartI
      
  def are_we_resetting_1(self):
    return self.reset_i
  
  def set_end(self):
    self.end = True
    #this will initiate shutdown procedure; a bit hacky, but this is how we do thing around here
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)  
    s.connect(("127.0.0.1", self.port))  # FIXME This may crash with connection refused error
    try:
      s.close()
    except:
      pass
  
  def form_message(self, info, typ):
    top_element = etree.Element("i-message", type = typ)
    for key, val in info.items():
      tag = etree.Element(key)
      tag.text = val
      top_element.append(tag)
    return etree.tostring(top_element, pretty_print=False)
  
  def clientthread(self, conn, addr):
    if self.end or self.reset_i:
      return
    with self.clients_lock:
      self.num_of_clients+=1
    try:
      line = ""
      #timeout = 10
      starting_tag_id = -1
      closing_tag_id = -1
      while (starting_tag_id < 0 or closing_tag_id < 0):
        line = line + conn.recv(self.recv_buffer_size).decode()
        starting_tag_id = line.find("<i-message ")
        closing_tag_id = line.find("</i-message>")
        if starting_tag_id >= 0 and closing_tag_id >= 0:
          break
      if starting_tag_id < 0 or closing_tag_id < 0:
        conn.send(self.form_message({"status":"ERR"}, "comm_response"))
        conn.close()
        with self.clients_lock:
          self.num_of_clients-=1
        return
      original_xml = None
      try:
        original_xml = etree.fromstring(line, self.parser)  
      except:
        log(self.logger, 'Line: {0}.'.format(line))
        conn.send(self.form_message({"status":"ERR"}, "comm_response"))
        conn.close()
        with self.clients_lock:
          self.num_of_clients-=1
        return
      if original_xml.attrib["type"] == "i_alive":
        conn.send(self.form_message({"status":"OK"}, "comm_response"))
      elif original_xml.attrib["type"] == "conf_request":
        if os.path.exists("conf/conf.xml"):
          fd = open("conf/conf.xml", "r")
          conf_xml = None
          try:
            conf_xml = etree.parse(fd)
            msg = self.form_message({"status":"OK"}, "comm_response")
            msg_node = etree.fromstring(msg)
            msg_node.append(conf_xml.getroot())
            conn.send(etree.tostring(msg_node, pretty_print=False))
          except:
            log_exception(self.logger, 'Parsing failed:')
            conn.send(self.form_message({"status":"ERR"}, "comm_response"))
            #conn.close()
          fd.close()
        else:
          log(self.logger, 'Conf file doesn\'t exist.', sev=logging.ERROR)
          conn.send(self.form_message({"status":"ERR"}, "comm_response"))
        #conn.close()
      elif original_xml.attrib["type"] == "conf":
        fd = open("conf/conf.xml", "w")
        fd.write(etree.tostring(original_xml.find("conf"), pretty_print=True).decode())
        fd.close()
        log(self.logger, 'User with IP ' + addr + " has uploaded the following conf:\n" + etree.tostring(original_xml.find("conf"), pretty_print=True).decode()) 
        self.reset_i = True
        log(self.logger, 'reset_i due to type==conf')
      elif original_xml.attrib["type"] == "i_reset":
        #fd = open("conf/conf.xml", "w")
        #fd.write(etree.tostring(original_xml.find("conf"), pretty_print=True))
        #fd.close()
        self.reset_i = True
        log(self.logger, 'reset_i due to type==i_reset')
      elif original_xml.attrib["type"] == "download_log":
        tdyy = date.today().strftime("%Y-%m-%d")
        tdyy_parts = tdyy.split("-")
        for i in range(1, len(tdyy_parts)):
          if len(tdyy_parts[i]) < 2:
            tdyy_parts[i] = "0" + tdyy_parts[i]
        tdyy = "-".join(tdyy_parts)
        try:
          fd = open("log/" + tdyy + "_" + original_xml.find("module_name").text + ".log", "r")
          log_txt = fd.read()
          fd.close()
          encoded = base64.b64encode(log_txt.encode())
          conn.send(self.form_message({"status":"OK", "content":encoded}, "comm_response"))
        except:
          conn.send(self.form_message({"status":"ERR"}, "comm_response"))
      elif original_xml.attrib["type"] == "download_swmstat":
        try:
          fd = open("log/swmstat.xml")
          log_txt = fd.read()
          fd.close()
          encoded = base64.b64encode(log_txt.encode())
          conn.send(self.form_message({"status":"OK", "content":encoded}, "comm_response"))
        except:
          log_exception(self.logger, 'Failed to send OK:')
          conn.send(self.form_message({"status":"ERR"}, "comm_response"))
        #self.reset_i = True
      conn.close()
    except:
      log_exception(self.logger, 'General exception:')
    with self.clients_lock:
      self.num_of_clients-=1
      
  def run(self):
    self.alive = True
    self.end = False
    try:
      self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM) 
      self.sock.bind(('', self.port))
      self.sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
      self.sock.listen(self.backlog)
    except:
      log_exception(self.logger, 'Fatal error:')
      self.alive = False
      return
    try:
      while  True:
        conn, addr = self.sock.accept()
        if self.end:
          while True:
            with self.clients_lock:
              if self.num_of_clients == 0:
                try:
                  conn.close()
                except:
                  pass
                self.sock.close()
                self.alive = False
                log(self.logger, 'Thread closed correctly.', sev=logging.DEBUG)
                return
            self.zzzzz(0.5)
        threading.Thread(target=self.clientthread, args=(conn, addr[0])).start()
    except:
      log_exception(self.logger, 'Fatal error:')
      self.sock.close()
    self.alive = False

        

      
      
    
    #i_message = original_xml 
  
    